import argparse
import ipaddress
import sys

import nettools.sos_infrastructure as sos
from nettools._version import __version__


def ipcalc_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument("-s", "--sans", help="Use sans-parsable output.", action="store_true")
    parser.add_argument("--dhcp", help="Get IPv6 address in the DHCP range.", action="store_true")

    parser.add_argument('--version', action='version', version='%(prog)s ' + __version__)

    commands = parser.add_subparsers(dest='command', help='commands')
    commands.required = True
    commands.add_parser('list', help='list available networks')

    v4to6_parser = commands.add_parser("4to6")
    v4to6_parser.add_argument("IPv4")

    v6to4_parser = commands.add_parser("6to4")
    v6to4_parser.add_argument("IPv6")
    return parser


def cli():
    parser = ipcalc_parser()
    args = parser.parse_args()

    if args.command == 'list':
        print(sos.subnets)
        return 0
    elif args.command == '4to6':
        try:
            v4 = ipaddress.IPv4Address(args.IPv4)
        except ipaddress.AddressValueError:
            print("Invalid IPv4 passed as argument", file=sys.stderr)
            return 1

        try:
            v6 = sos.subnets.ip4to6(ipv4=v4, sans=args.sans, dhcp=args.dhcp)
        except sos.NotStaticSubnet:
            print("ERROR: The subnet doesn't support static IP assignment.")
            return 1
        except sos.NotDHCPSubnet:
            print("ERROR: The subnet doesn't support DHCP IP assignment.")
            return 1

        if not v6:
            print("Failed to find subnet data for address: {}".format(args.IPv4), file=sys.stderr)
            return 1
        else:
            print(v6)
            return 0

    elif args.command == '6to4':
        raise NotImplementedError("The IPv6 to IPv4 conversion is not implemented.")
    else:
        parser.print_usage()
        return 1
